#ifndef CLIENTORBMANAGER_H_HEADER_INCLUDED_C0344A35
#define CLIENTORBMANAGER_H_HEADER_INCLUDED_C0344A35

#include <string>
#include <iostream>
#include "NamingServiceFacade.h"

class NamingServiceFacade;

// This template class establishes a connetion with the CORBA ORB and
// establishes a connection with the CORBA server that implements the interface
// used by the CORBA client.
//##ModelId=3FC51F3303DC
template <
    // Class name of the IDL interface object.
    typename CorbaObjType, 
    // The _var class name of the IDL interface. If the interface class name is
    // Navigation, then the _var class name is fromed be appending _var to the
    // class name: Navigation_var. The _var class is generated by the IDL
    // compiler. The _var class acts as a smart pointer that wraps the
    // underlying type.
    typename CorbaObjVarType>
class ClientOrbManager
{

  public:
    //##ModelId=3FC52FE900A0
    ClientOrbManager()
    {
      clientPtr = CORBA_OBJECT_NIL ;
      namingService = new NamingServiceFacade ;
    }

    //##ModelId=3FC52FE900DF
    virtual ~ClientOrbManager()
    {
      delete namingService ;
    }

    // This operation initializes the client processes connect to the ORB and
    // uses the Naming Service to get the IOR of the CORBA Server the
    // implements the CORBA interface used by the client.
    //##ModelId=3FC532F800B0
    bool initOrb(int argc, char** argv)
    {
        try{
            orb = CORBA::ORB_init(argc, argv, "Raytheon-Netfires");
        }
        catch(...){
            std::cout<<"Cannot initialize ORB for process"<<argv[0]<<std::endl ;
            return false ;
        }
        // get Naming Service object
        namingService->resolveNamingServiceObj(orb);

        return true ;
    }

    //##ModelId=3FC5359403DC
    CorbaObjVarType get_clientObj(const std::string& bindingName)
    {
        //resolve server name with Naming Service and create client object
        CORBA::Object::Object * serverIor = namingService->getServerObj(bindingName);
        if(serverIor == CORBA_OBJECT_NIL)
	  {
	    std::cout<<"Failed to get resolve server name with Naming Service"<<std::endl ;
            return CORBA_OBJECT_NIL ;
	  }
        clientPtr = CorbaObjType::_narrow(serverIor);
        return clientPtr;
    }

  private:
    // This attribute contains the CORBA server IOR that is needed by the
    // client to access opertions on the IDL interface.
    //##ModelId=3FC534750246
    CorbaObjVarType clientPtr;

    //##ModelId=3FC5570E01F8
    NamingServiceFacade *namingService;
    //##ModelId=3FCBC3B602FC
    CORBA::ORB_ptr orb;

};



#endif /* CLIENTORBMANAGER_H_HEADER_INCLUDED_C0344A35 */
