// File: ClientToolProc.cc

#include <pthread.h>
#include <iostream>
#include <utility>
#include <vector>

// ORBit2 includes
#include "orbitcpp/orb-cpp/orbitcpp.h"

#include "ServerOrbManager.h"
#include "ClientInterface-cpp-common.h"
#include "ClientInterface-impl.h"


int main (int argc, char** argv)
{
  // Set up the connection to the Client interface

  // Check if CORBA server binding name was passed as a command line
  // parameter with the format of -n "binding name" or
  // -name "binding name"
  std::vector<std::string> v;
  std::vector<std::string>::iterator vi;

  for (int i = 0; i < argc; i++)
  {
    if (argv[i] == NULL)
    {
      // end of arguments
      break;
    }
    // save the argument strings in the vector
    v.push_back(argv[i]);
  }

  // Look for the parameter in the vector
  const std::string p1Name = "-n";
  const std::string p2Name = "-name";
  std::string name = "/ClientInterface/";
  for (vi = v.begin(); vi != v.end(); vi++)
  {
    if ((*vi == p1Name) || (*vi == p2Name))
    {
      // found the name flag
      if ((vi + 1) == v.end())
      {
        // No string after the flag - use the default name
        break;
      }
      else if ((*(vi + 1))[0] == '-')
      {
        // Found another flag - use the default name
        break;
      }
      else
      {
        // Save the context name string
        name = *(vi + 1);
        break;
      }
    }
  }

  #ifdef DEBUG
  std::cout << "name = " << name << std::endl;
  #endif

  // Initialize the connection to the ORB server
  ServerOrbManager<NLCA::ClientInterface, NLCA::ClientInterface_var, NLCA::ClientInterface_impl>
    serverOrb ;
  if(serverOrb.initOrb(argc, argv, name) == true)
  {
    // Enter the ORB's event loop
    serverOrb.runOrb();
  }
  else
  {
    std::cout<<"Failed in Orb init"<<std::endl ;
    exit(1) ;
  }

  return 0 ;

} // end main

