#include <iostream>
#include <cstdio>
#include <vector>
#include "NamingServiceFacade.h"


//#define UNIT_TEST_REDCLIENT

//##ModelId=3FC5423C03BD
NamingServiceFacade::NamingServiceFacade()
{
    #ifdef UNIT_TEST_REDCLIENT
    std::cout << "NamingServiceFacade::NamingServiceFacade() called" << std::endl;
    #endif

    serverCobj = CORBA_OBJECT_NIL;
    ptrServerObj = CORBA_OBJECT_NIL;
}

NamingServiceFacade::~NamingServiceFacade()
{
    if (nameComp != NULL)
    {
      delete [] nameComp;
    }
}

//##ModelId=3FC539FE0004
CORBA::Object::Object * NamingServiceFacade::getServerObj(const std::string& bindingName)
{
    // Parse the CORBA server's name for the call to resolve the name
    parseName(bindingName);

    // Get the CORBA_Object for the client's server
    // from the naming service

    CORBA_Environment  ev[1];
    CORBA_exception_init(ev);

    #ifdef UNIT_TEST_REDCLIENT
    std::cout << "calling CosNaming_NamingContext_resolve()" << std::endl;
    std::cout << "Name Obj =" << namingServiceObj << std::endl; 
    #endif

    CORBA_Object & NamingSeviceCobj = namingServiceObj->_orbitcpp_cobj();
    serverCobj = CosNaming_NamingContext_resolve(NamingSeviceCobj,
                                                    &serverName,
                                                    ev);

    #ifdef DEBUG
    std::cout << "returned from CosNaming_NamingContext_resolve()" << std::endl;
    std::cout << "serverCobj = " << serverCobj << std::endl;
    #endif /* DEBUG */

    if(ev->_major != CORBA_NO_EXCEPTION){
      #ifdef DEBUG
      std::cout << "ev->_id = " << CORBA_exception_id(ev) << std::endl;
      #endif

      return CORBA_OBJECT_NIL;// Failed to resolve  
    }

    // convert from C style CORBA_Object to C++ Object
    #ifdef DEBUG
    std::cout << "Call CORBA::Object::_orbitcpp_wrap(serverCobj,false)" << std::endl;
    #endif /* DEBUG */

    ptrServerObj = CORBA::Object::_orbitcpp_wrap(serverCobj,false);

    #ifdef DEBUG
    std::cout << "ptrServerObj = " << ptrServerObj  << std::endl;
    #endif /* DEBUG */
    if(!ptrServerObj)
    {
        return CORBA_OBJECT_NIL;
    }

    return ptrServerObj;
}

//##ModelId=3FC53BDF0275
int NamingServiceFacade::resolveNamingServiceObj(CORBA::ORB_ptr orb)
{
    char filebuffer[1024];
    FILE * testfile;
    char * NamingIOR;

    testfile = fopen("/etc/NamingService.ior","r");
    if(testfile == NULL){
      std::cout<<"file NamingService.ior failed to open. errno="
               << errno << " : " << strerror(errno)
               <<std::endl ;
	exit(1) ;
    }
    fgets(filebuffer,1023,testfile);
    NamingIOR = g_strdup(filebuffer);
    fclose(testfile);

    
#ifdef DEBUG
      std::cout << "here comes the IOR: " << NamingIOR << " done" << std::endl;
#endif /* DEBUG */

    try{
        namingServiceObj = orb->string_to_object(NamingIOR); // C++ Object returned
    }
    catch(...){
        #ifdef UNIT_TEST_REDNAMING
        std::cout << "Bad format of stringified reference to Naming Service" << std::endl;
        #endif
        exit(1);
    }

    if(CORBA::is_nil(namingServiceObj)){
        #ifdef UNIT_TEST_REDNAMING
        std::cout << "passed reference for Naming Service is nil" << std::endl;
        #endif
        exit(1);
    }

    namingServiceCobj = _orbitcpp::cobj_guarded(namingServiceObj); // standard C CORBA_Object returned
    return 0;


}

//##ModelId=3FC542C302C3
void NamingServiceFacade::parseName(const std::string& bindingName)
{
    #ifdef UNIT_TEST_REDCLIENT
    std::cout << "NamingServiceFacade::parseName(const std::string& bindingName): ";
    std::cout << "bindingName = " << bindingName << std::endl;
    #endif

    std::vector<CosNaming_NameComponent> contextLevels;
    std::string context;

    const std::string ContextDelimiter = "/";
    const std::string KindDelimiter = ".";

    std::string s = bindingName;
    #ifdef UNIT_TEST_REDCLIENT
    std::cout << "s = " << s << std::endl;
    std::cout << "s.length() = " << s.length() << std::endl;
    #endif
    while(s.length() > 0)
    {
      // Find the next context in the string
      std::string::size_type i = s.find(ContextDelimiter);
      #ifdef UNIT_TEST_REDCLIENT
      std::cout << "i = " << i << std::endl;
      #endif
      if (i == std::string::npos)
      {
        // No context delimiter in the string - found the last context
        context = s;
        s.erase();
        #ifdef UNIT_TEST_REDCLIENT
        std::cout << "last context: context = " << context << std::endl;
        std::cout << "s = " << s << std::endl;
        #endif
      }
      else if (i == 0)
      {
        // Found delimiter at front of the string - remove it
        s.erase(i, 1);
        #ifdef UNIT_TEST_REDCLIENT
        std::cout << "remove /: s = " << s << std::endl;
        #endif
        continue;
      }
      else
      {
        // Found the end of a context
        context = s.substr(0, i);
        s.erase(0, i);
        #ifdef UNIT_TEST_REDCLIENT
        std::cout << "found end of context: context = ";
        std::cout << context << std::endl;
        std::cout << "s = " << s << std::endl;
        #endif
      }

      // Split the context into id and kind
      std::string::size_type j = context.find(KindDelimiter);
      #ifdef UNIT_TEST_REDCLIENT
      std::cout << "j = " << j << std::endl;
      #endif
      if (j == std::string::npos)
      {
        // The optional kind value was not provided
        CosNaming_NameComponent nc;
        // nc.id = const_cast<CORBA_char*>(context.c_str());
        nc.id = CORBA::string_dup(context.c_str());
        nc.kind = CORBA::string_dup("");
        #ifdef UNIT_TEST_REDCLIENT
        std::cout << "no kind value: nc<<" << nc.id << ", " << nc.kind << ">>" << std::endl;
        #endif
        contextLevels.push_back(nc);
      }
      else if (j == 0 || j == (context.size() - 1))
      {
        // Error - delimiter is at the front or end of the context
        std::cout << "Error in NamingServiceFacade::parseName(): ";
        std::cout << "in binding name ";
        std::cout << bindingName << " the context ";
        std::cout << context << " contains a period (.) without ";
        std::cout << "a kind value" << std::endl;

        exit(1);
      }
      else
      {
        // Verify there is only one kind delimiter in the string
        std::string::size_type m = context.rfind(KindDelimiter);
        if (j != m)
        {
          // Error - more than one kind delimitor in string
          std::cout << "Error in NamingServiceFacade::parseName(): ";
          std::cout << "there is more than one period in the context = ";
          std::cout << context << std::endl;

          exit(1);
        }

        // Separate the id and kind values
        CosNaming_NameComponent nc;
        // substr - 1st parameter is the index of the start of the
        // substring and 2nd parameter in the number of characters
        // in the substring
        nc.id = CORBA::string_dup
          ((context.substr(0, j)).c_str());
        nc.kind = CORBA::string_dup
          ((context.substr((j + 1), (context.size() - j - 1))).c_str());
        #ifdef UNIT_TEST_REDCLIENT
        std::cout << "id and kind values: nc<<" << nc.id << ", " << nc.kind << ">>" << std::endl;
        #endif
        contextLevels.push_back(nc);
      }
    } // end while

    // Copy the context levels to an array
    #ifdef UNIT_TEST_REDCLIENT
    std::cout << "contextLevels.size() = " << contextLevels.size() << std::endl;
    #endif
    if (contextLevels.size() == 0)
    {
      // Error - no contexts were found in the input string
      std::cout << "Error in NamingServiceFacade::parseName(): ";
      std::cout << "in binding name ";
      std::cout << bindingName << " - unable to find any contexts";
      std::cout << std::endl;

      exit(1);
    }

#ifdef DONNA
    if (nameComp != NULL)
    {
      // Parser was called previously - release memory to avoid
      // a memory leak
      #ifdef UNIT_TEST_REDCLIENT
      std::cout << "free memory pointed to by nameComp" << std::endl;
      #endif
      delete [] nameComp;
    }
#endif /* DONNA */

    nameComp = new CosNaming_NameComponent[contextLevels.size()];
    std::vector<CosNaming_NameComponent>::iterator k;
    int l;
    for (k = contextLevels.begin(), l = 0; k != contextLevels.end(); k++, l++)
    {
      nameComp[l].id = (*k).id;
      nameComp[l].kind = (*k).kind;
      #ifdef UNIT_TEST_REDCLIENT
      std::cout << "nameComp[" << l << "].id = " << nameComp[l].id << std::endl;
      std::cout << "nameComp[" << l << "].kind = " << nameComp[l].kind << std::endl;
      #endif
    }

    serverName._length = contextLevels.size();
    serverName._maximum = contextLevels.size();
    serverName._buffer = nameComp;
    serverName._release = CORBA_FALSE; 
}

//##ModelId=3FCCCCF10009
bool NamingServiceFacade::registerWithNamingService(CORBA_Object & ServerCobj,
    const std::string& bindingName)
{
    CORBA_Environment  ev[1];
    CORBA_exception_init(ev);
    parseName(bindingName);
    // register (bind) Nav Msg server with Names Service
    CosNaming_NamingContext_bind( namingServiceCobj,// corba object of Naming Service
                                  &serverName,
                                  ServerCobj,// Corba object of Nav server to register with Naming Service
                                  ev);
    
    #ifdef DEBUG
    std::cout << "ev->_major = " << ev->_major << std::endl;
    std::cout << "\tCORBA_NO_EXCEPTION = " << CORBA_NO_EXCEPTION << std::endl;
    std::cout << "\tCORBA_USER_EXCEPTION= " << CORBA_USER_EXCEPTION<< std::endl;
    std::cout << "\tCORBA_SYSTEM_EXCEPTION= " << CORBA_SYSTEM_EXCEPTION<< std::endl;
    #endif

    if(ev->_major != CORBA_NO_EXCEPTION){
      std::cout<<"Failed to bind server with Naming Service"<<std::endl;

      #ifdef DEBUG
      std::cout << "ev->_id = " << CORBA_exception_id(ev) << std::endl;
      #endif

      exit(1);
    }
    return true;
}

